import enum

class SearchDiscussionsOrderBy(enum.Enum):
  SEARCH_DISCUSSIONS_ORDER_BY_UNSPECIFIED = 0
  SEARCH_DISCUSSIONS_ORDER_BY_LAST_TOPIC_COMMENT_DATE = 1

class SearchDiscussionsDocumentType(enum.Enum):
  SEARCH_DISCUSSIONS_DOCUMENT_TYPE_UNSPECIFIED = 0
  SEARCH_DISCUSSIONS_DOCUMENT_TYPE_COMMENT = 1
  SEARCH_DISCUSSIONS_DOCUMENT_TYPE_TOPIC = 2
  SEARCH_DISCUSSIONS_DOCUMENT_TYPE_WRITE_UP = 3

class SearchDiscussionsSourceType(enum.Enum):
  SEARCH_DISCUSSIONS_SOURCE_TYPE_UNSPECIFIED = 0
  SEARCH_DISCUSSIONS_SOURCE_TYPE_COMPETITION = 1
  SEARCH_DISCUSSIONS_SOURCE_TYPE_DATASET = 2
  SEARCH_DISCUSSIONS_SOURCE_TYPE_KERNEL = 4
  SEARCH_DISCUSSIONS_SOURCE_TYPE_SITE_FORUM = 5
  SEARCH_DISCUSSIONS_SOURCE_TYPE_COMPETITION_SOLUTION = 6
  SEARCH_DISCUSSIONS_SOURCE_TYPE_MODEL = 7
  SEARCH_DISCUSSIONS_SOURCE_TYPE_WRITE_UP = 8

class SearchDiscussionsTopicType(enum.Enum):
  SEARCH_DISCUSSIONS_TOPIC_TYPE_UNSPECIFIED = 0
  SEARCH_DISCUSSIONS_TOPIC_TYPE_TOPICS = 1
  SEARCH_DISCUSSIONS_TOPIC_TYPE_WRITE_UPS = 2

class WriteUpInclusionType(enum.Enum):
  WRITE_UP_INCLUSION_TYPE_UNSPECIFIED = 0
  WRITE_UP_INCLUSION_TYPE_EXCLUDE = 1
  r"""
  Only ForumTopics will be included, while
  WriteUps will be excluded
  """
  WRITE_UP_INCLUSION_TYPE_INCLUDE = 2
  """WriteUps and ForumTopics will be included"""
  WRITE_UP_INCLUSION_TYPE_ONLY = 3
  r"""
  Only WriteUps will be included, while
  ForumTopics will be excluded
  """

