import enum

class UserAchievementType(enum.Enum):
  USER_ACHIEVEMENT_TYPE_UNSPECIFIED = 0
  USER_ACHIEVEMENT_TYPE_COMPETITIONS = 1
  USER_ACHIEVEMENT_TYPE_NOTEBOOKS = 2
  USER_ACHIEVEMENT_TYPE_DISCUSSION = 3
  USER_ACHIEVEMENT_TYPE_DATASETS = 4

