import enum

class SearchCompetitionsOrderBy(enum.Enum):
  SEARCH_COMPETITIONS_ORDER_BY_UNSPECIFIED = 0
  SEARCH_COMPETITIONS_ORDER_BY_DEADLINE = 1
  SEARCH_COMPETITIONS_ORDER_BY_PRIZE_VALUE = 2
  SEARCH_COMPETITIONS_ORDER_BY_TEAM_RANK = 3
  SEARCH_COMPETITIONS_ORDER_BY_LAST_SUBMISSION_DATE = 4

class SearchCompetitionsProfileVisibility(enum.Enum):
  SEARCH_COMPETITIONS_PROFILE_VISIBILITY_ANY = 0
  SEARCH_COMPETITIONS_PROFILE_VISIBILITY_VISIBLE = 1
  SEARCH_COMPETITIONS_PROFILE_VISIBILITY_HIDDEN = 2

class SearchCompetitionsRole(enum.Enum):
  SEARCH_COMPETITIONS_ROLE_ANY = 0
  SEARCH_COMPETITIONS_ROLE_HOST = 1
  SEARCH_COMPETITIONS_ROLE_PARTICIPANT = 2
  SEARCH_COMPETITIONS_ROLE_PARTICIPANT_ONLY = 3
  """Excludes competitions user hosted, even if they are also a participant"""

class SearchCompetitionsStatus(enum.Enum):
  SEARCH_COMPETITIONS_STATUS_ANY = 0
  SEARCH_COMPETITIONS_STATUS_ACTIVE = 1
  SEARCH_COMPETITIONS_STATUS_COMPLETE = 2
  SEARCH_COMPETITIONS_STATUS_UNLAUNCHED = 3

