import enum

class SearchModelsOrderBy(enum.Enum):
  MODELS_SEARCH_ORDER_BY_UNSPECIFIED = 0
  MODELS_SEARCH_ORDER_BY_DOWNLOAD_COUNT = 1
  MODELS_SEARCH_ORDER_BY_NOTEBOOK_COUNT = 2
  MODELS_SEARCH_ORDER_BY_PUBLISH_DATE = 3

